"""
    Note: Strictly speaking, these are not unit tests - they're
    acceptance tests. They provide a set of tests to validate
    morbid, messagequeue, and mqsecurity and how they work together.
    
    The idea is that these tests can also be used as the basis for
    a set of tests designed to exercise a complete morbid configuration.
"""

import unittest

import logging

from morbid import messagequeue
from morbid import mqsecurity
from morbid import StompProtocol, StompFactory

import stomper

from zope.interface import implements

from twisted.trial import unittest
from twisted.internet import reactor, protocol, defer, error
from twisted.internet.protocol import Protocol, ReconnectingClientFactory, ClientCreator

from twisted.protocols import loopback

# Useful constants for the tests

uall = '/topic/users/user2/user3/c/r/w/'
uread = '/topic/users/r/'
uwrite = '/topic/users/w/'
ucreate = '/topic/users/c/'

qall = '/queue/users/user2/user3/c/r/w/'
qread = '/queue/users/r/'
qwrite = '/queue/users/w/'

pall = '/pubsub/users/c/r/w/'
ptwo = '/pubsub/users/c/r/w/'

test_message_1 = "This is a test"
test_message_2 = "This is line 2"
test_header_1 = {'transaction':'atran'}
stomper.utils.log_init(logging.CRITICAL)


def queue_rights(group_list, qname):
    r = set()
    for g in group_list:
        if g == 'admin':
            r = set(['c','r','w'])
            break
        if g not in qname:
            continue
        for type in ['/c/', '/r/', '/w/']:
            if type in qname:
                r.add(type[1])
    return r


class MsgError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

        
class Expects(object):
    def __init__(self):
        self.emsg = []
        self.deferred = []
        
    def add_emsg(self, emsg, d1):
        self.emsg.append(emsg)
        self.deferred.append(d1)

    def check_rmsg(self, rmsg):
        if (rmsg['body'] in self.emsg):
            i = self.emsg.index(rmsg['body'])
            if (i > 0):
                print
                print "Warning: message received out of sequence expected"
                print rmsg
                print "expecting :"
                print self.emsg[0]
                print
            self.emsg.pop(i)
            d1 = self.deferred.pop(i)
            d1.callback(rmsg)
        else:
            print
            print "Unexpected message:"
            print "expecting one of -", self.emsg
            print "received -",rmsg['body']
            raise MsgError("Unexpected message received:\n"+repr(rmsg)+"\n")


class StompClientProtocol(Protocol, stomper.Engine):

    def __init__(self, username='xxx', password='yyy'):
        stomper.Engine.__init__(self)
        self.username = username
        self.password = password
        self.counter = 1
        self.expects = Expects()

    def ack(self, msg):
        self.expects.check_rmsg(msg)
        self.log.info("SENDER - received: %s " % msg['body'])

    def send(self, msg, mq):
        f = stomper.Frame()
        f.unpack(stomper.send(mq, msg))
        self.counter += 1        
        self.transport.write(f.pack())

    def subscribe(self, qname):
        self.transport.write(stomper.subscribe(qname))

    def connectionMade(self):
        cmd = stomper.connect(self.username, self.password)
        self.transport.write(cmd)

    def connected(self, msg):
        self.sessionId = msg['headers']['session']
    
    def error(self, msg):
        #print 'error msg =', msg
        self.expects.check_rmsg(msg)
        self.log.info("SENDER - received: %s " % msg['body'])

    def dataReceived(self, data):
        all_msgs = data.split('\x00\n')
        for a_msg in all_msgs:
            if a_msg:
                #print 'dr - msg =', a_msg
                msg = stomper.unpack_frame(a_msg)
                returned = self.react(msg)
                if returned:
                    self.transport.write(returned)


class MorbidTestCase(unittest.TestCase):
    def setUp(self):
        self.timeout = 5
        self.log = logging.getLogger("sender")
        self.mqm = messagequeue.MessageQueueManager()
        self.mqs = mqsecurity.MQPortal(self.mqm, None)
        self.mqm.set_queue_rights(queue_rights)
 
        self.factory = StompFactory(mqm=self.mqm)
        self.factory.mq_portal = self.mqs
        self.serverProtocol = self.factory.buildProtocol(None)
        self.serverProtocol.log = self.log
        self.client = StompClientProtocol('users','users')
        self.client.log = self.log
        self.loopbackBody = loopback.loopbackAsync(self.serverProtocol, self.client)

    def waitForResponse(self, client, msg):
        d1 = self.listenForResponse(client, msg)
        client.send(msg, self.dest)
        return d1

    def listenForResponse(self, client, msg):
        d1 = defer.Deferred()
        d1.addCallback(self.gotResponse)
        d1.addErrback(self.msgError)
        client.expects.add_emsg(msg, d1)
        return d1
    
    def gotResponse(self, msg):
        if 'done' in msg['body']:
            self.client.transport.loseConnection()
        #return er == msg['body']
            
    def msgError(self, *args):
        return args[0]

    def waitForErrorMsg(self, client, msg, err):
        d1 = self.listenForErrorMsg(client, err)
        client.send(msg, self.dest)
        return d1
    
    def listenForErrorMsg(self, client, err):
        d1 = defer.Deferred()
        d1.addCallback(self.gotError)
        d1.addErrback(self.msgError)
        client.expects.add_emsg(err, d1)
        return d1
        
    def gotError(self, msg):
        self.assertEquals(msg['cmd'], 'ERROR')


class BasicMorbidServerTestCase(MorbidTestCase):
    def test_SendTopicText(self):
        self.dest = uall
        self.client.subscribe(self.dest)
        d1 = self.waitForResponse(self.client, test_message_1)
        d2 = self.waitForResponse(self.client, test_message_2)
        d3 = self.waitForResponse(self.client, "done")
        dl = defer.DeferredList([d1, d2, d3])
        return dl
    
    def test_SendQueueTest(self):
        self.dest = qall
        d1 = self.waitForResponse(self.client, test_message_1)
        d2 = self.waitForResponse(self.client, test_message_2)
        d3 = self.waitForResponse(self.client, "done")
        self.client.subscribe(self.dest)
        dl = defer.DeferredList([d1, d2, d3])
        return dl

    def test_checkCreateRights(self):
        d1 = self.client.subscribe(uall)
        return d1
    
    def test_checkNoCreateRights(self):
        self.dest = uread
        d1 = self.waitForErrorMsg(self.client, 
                                  test_message_1,
                                  "Not authorized to create queue")
        return d1

    def test_checkNoWriteRights(self):
        self.dest = ucreate
        d1 = self.waitForErrorMsg(self.client,
                                  test_message_2,
                                  "Not authorized to write to queue")
        return d1

class TwoClientServerTestCase(MorbidTestCase):
    def setUp(self):
        MorbidTestCase.setUp(self)
        self.serverProtocol2 = self.factory.buildProtocol(None)
        self.serverProtocol2.log = self.log
        self.adminClient = StompClientProtocol('admin','admin')
        self.adminClient.log = self.log
        self.loopbackBody = loopback.loopbackAsync(self.serverProtocol2, 
                                                   self.adminClient)
   
    def test_checkReadRights(self):
        self.dest = qread
        self.adminClient.send(test_message_1, self.dest)
        d1 = self.listenForResponse(self.client, test_message_1)
        self.client.subscribe(self.dest)
        return d1

    def test_checkNoReadRights(self):
        self.dest = qwrite
        self.adminClient.send(test_message_1, self.dest)
        d1 = self.listenForErrorMsg(self.client, 
                                    "Not authorized to read queue")
        self.client.subscribe(self.dest)
        return d1
    
    def test_checkWriteRights(self):
        self.dest = qwrite
        d1 = self.listenForResponse(self.adminClient, test_message_2)
        self.adminClient.subscribe(self.dest)
        self.client.send(test_message_2, self.dest)
        return d1
    

class TwoListenersTestCase(MorbidTestCase):
    def setUp(self):
        MorbidTestCase.setUp(self)
        self.serverProtocol2 = self.factory.buildProtocol(None)
        self.serverProtocol2.log = self.log
        self.client2 = StompClientProtocol('user2','user2')
        self.client2.log = self.log
        self.loopbackBody = loopback.loopbackAsync(self.serverProtocol2, 
                                                   self.client2)

        self.serverProtocol3 = self.factory.buildProtocol(None)
        self.serverProtocol3.log = self.log
        self.client3 = StompClientProtocol('user3','user3')
        self.client3.log = self.log
        self.loopbackBody = loopback.loopbackAsync(self.serverProtocol3, 
                                                   self.client3)

    def test_pubsub(self):
        
        self.mqm.add_pubsub_child(self.serverProtocol, ptwo, uall+'2')
        self.mqm.add_pubsub_child(self.serverProtocol, ptwo, uall+'3')

        d2 = self.listenForResponse(self.client2, test_message_1)
        self.client2.subscribe(uall+'2')
        d3 = self.listenForResponse(self.client3, test_message_1)
        self.client3.subscribe(uall+'3')
        
        self.client.send(test_message_1, ptwo)

        dl = defer.DeferredList([d2, d3])
        return dl

    def test_pubsub_queue_multiple(self):
        
        self.mqm.add_pubsub_child(self.serverProtocol, ptwo, qall+'1')
        self.mqm.add_pubsub_child(self.serverProtocol, ptwo, qall+'2')
        self.mqm.add_pubsub_child(self.serverProtocol, ptwo, qall+'3')

        self.client.send(test_message_1, ptwo)
        self.client.send(test_message_2, ptwo)
        
        d11 = self.listenForResponse(self.client, test_message_1)
        d12 = self.listenForResponse(self.client, test_message_2)
        self.client.subscribe(qall+'1')

        d21 = self.listenForResponse(self.client2, test_message_1)
        d22 = self.listenForResponse(self.client2, test_message_2)
        self.client2.subscribe(qall+'2')

        d31 = self.listenForResponse(self.client3, test_message_1)
        d32 = self.listenForResponse(self.client3, test_message_2)
        self.client3.subscribe(qall+'3')
        
        dl = defer.DeferredList([d11, d12, d21, d22, d31, d32])
        return dl



import unittest
import os, sys
sys.path.insert(0, os.path.join('..','sample_auth'))
mqanyconf = __import__('mqanyconf')

"""
This is a copy of testmqfileconf, altered to test that all IDs have all access to all queues
"""

class ParmTestCase(unittest.TestCase):
    def setUp(self):
        self.parms = mqanyconf.Parms()
        self.do_group_rights = self.parms.get_group_access_rights()
        self.co = set('c')
        self.ro = set('r')
        self.wo = set('w')
        self.cr = set(['c', 'r'])
        self.cw = set(['c', 'w'])
        self.rw = set(['r', 'w'])
        self.crw = set(['r', 'w', 'c'])
        self.none = set()
        
    def testUnknownQueue(self):
        self.assertEquals(self.do_group_rights(['user'],'/topic/unknown'), self.crw)

    def testTextQueueUser(self):
        self.assertEquals(self.do_group_rights(['user'],'/topic/text/1'), self.crw)

    def testTextQueueAdmin(self):
        self.assertEquals(self.do_group_rights(['admin'],'/topic/text/1'), self.crw)
        
    def testHomeQueueUnknownUser(self):
        self.assertEquals(self.do_group_rights(['userx'],'/topic/home/1'), self.crw)
        
    def testAdminQueueAdmin(self):
        self.assertEquals(self.do_group_rights(['admin'],'/topic/admin/2'), self.crw)
        
    def testAdminQueueUser(self):
        self.assertEquals(self.do_group_rights(['user'],'/topic/admin/2'), self.crw)
        
    def testCommandQueueSite(self):
        self.assertEquals(self.do_group_rights(['site'],'/command/front'), self.crw)
        
    def testCommandQueueAdmin(self):
        self.assertEquals(self.do_group_rights(['admin'],'/command/front'), self.crw)
        
    def testCommandQueueUser(self):
        self.assertEquals(self.do_group_rights(['user'],'/command/front'), self.crw)
        
    def testGroupHomeQueue(self):
        self.assertEquals(self.do_group_rights(['user'],'/home/user/1'), self.crw)
        
    def testOtherGroupHomeQueue(self):
        self.assertEquals(self.do_group_rights(['admin'],'/home/user/2'), self.crw)
        
    def testMultipleGroups(self):
        self.assertEquals(self.do_group_rights(['admin', 'user'],'/topic/text/3'), self.crw)
        
    def testAdditiveRights(self):
        self.assertEquals(self.do_group_rights(['site','admin'],'/command/back'), self.crw)
        
    def testHomeListeningQueue(self):
        self.assertEquals(self.do_group_rights(['usr1', 'user'],'/queue/read/usr1'), self.crw)
        
    def testAdminHomeListeningQueue(self):
        self.assertEquals(self.do_group_rights(['admin'],'/queue/read/usr1'), self.crw)
        



"""
A configuration module must contain a definition of a Parm class.

The function checker_config takes no args, and returns a dictionary
with the information needed to assign a credentials checker.

The 'sec_type' member defines the type of security checker to use

The function group_config takes no args, and returns a function
that takes one arg (the username) and returns the list of groups
to which the user belongs.

This sample file uses ID, password and group information from a text
file.

The function get_group_access_rights takes no args, and returns a
function that takes two args (group name, queue name).

This second function returns the access levels that the group has
to that queue ('r', 'w', 'c')
r - read
w - write
c - create
"""
class Parms(object):
    def checker_config(self):
        return {
            'sec_type':'file',
            'filename':'userIdFile.txt',
            'cache':True
            }
    
    def read_id_file(self, name):
        userFile = open('userIdFile.txt','r')
        # A user is always a member of a group matching their name
        group = [name]
        for line in userFile:
            fields = line.split(":")
            if (name == fields[0]):
                group.append(fields[2])
        userFile.close()
        return group
        
    def group_config(self):
        return  self.read_id_file

    def group_rights(self, groups, queue):
        # The most permissive set of group permissions are assigned.
        # (Rights are additive across levels and groups)
        # Therefore, it is not possible to restrict rights based
        # on lower levels of the queue named hierarchy
        rights = set()
        for qname in self.queues:
            if queue.startswith(qname):
                for gname in groups:
                    if gname in self.queues[qname]:
                        for rname in self.queues[qname][gname]:
                            rights.add(rname)
        #Special case - grant all rights to a groups' home queue
        for gname in groups:
            if queue.startswith('/home/'+gname):
                rights = set(('c', 'r', 'w'))
                break
        #Another special case - grant access to their private home listening queue
        if queue.startswith('/queue/read/'+groups[0]):
            rights = set('r')
            
        return rights

    def get_group_access_rights(self):
        self.queues = {
                  '/topic/text': {'user': ('r', 'w'),
                                  'admin': ('c', 'r', 'w')},
                  '/topic/home': {'user': ('c', 'r', 'w'),
                                  'admin': ('c', 'r', 'w')},
                  '/topic/admin': {'admin':('c', 'r', 'w')},
                  '/queue/read': {'admin':('c', 'w'),
                                  'site':('c', 'w')},
                  '/command': {'site': ('c', 'w'),
                               'admin':('r', 'w')}
                }
        return self.group_rights



class Parms(object):
    """
    A configuration module must contain a definition of a Parm class.

    The function checker_config takes no args, and returns a dictionary
    with the information needed to assign a credentials checker.

    The 'sec_type' member defines the type of security checker to use

    The function group_config takes no args, and returns a function
    that takes one arg (the username) and returns the list of groups
    to which the user belongs.

    This sample file uses ID, password and group information from a text
    file.

    The function get_group_access_rights takes no args, and returns a
    function that takes two args (group name, queue name).

    This second function returns the access levels that the group has
    to that queue ('r', 'w', 'c')
    r - read
    w - write
    c - create

    This configuration module is intended to be the default.
    It will accept all credentials and provide unrestricted access
    to all queues.
    """
    def checker_config(self):
        return {'sec_type':'any'}

    def group_config(self):
        return lambda name: [name]
    
    def get_group_access_rights(self):
        return self.get_group_rights
    
    def get_group_rights(self, groups, queue):
        return set(['c','r','w'])



class QueueError(Exception):
    
    def __init__(self, error_code):
        self.code = error_code
        
    def __str__(self):
        return repr(self.code)


class MessageQueueRoot(object):
    def __init__(self, name):
        self.name = name
        self.subscribers = []
        self.id = 0

    def subscribe(self, proto):      
        #print "mqr - subscribe - proto =", proto
        if proto not in self.subscribers:
            self.subscribers.append(proto)
            
    def unsubscribe(self, proto):
        if proto in self.subscribers:
            self.subscribers.remove(proto)
            
    def empty(self):
        return not bool(self.subscribers)

    def prep_headers(self, headers):
        self.id += 1
        id = self.name + '_' + str(self.id)
        headers.update({'destination': self.name, 'message-id': id})
        return headers

    def prep_message(self, message):
        #print "message to prep", message
        if isinstance(message, tuple):
            headers, body = message
        else:
            headers, body = {}, message
        message = (self.prep_headers(headers), body)
        return message


class Queue(MessageQueueRoot):
    """
    A queue sends a message to any one listener
    """
    
    def __init__(self, name):
        MessageQueueRoot.__init__(self, name)
        self.messages = []

    def subscribe(self, proto):
        MessageQueueRoot.subscribe(self, proto)
        while self.messages:
            message = self.messages.pop(0)
            proto.send(self.prep_message(message))
            
    def send(self, message):
        """ Calls the send function for each attached protocol """
        if not self.subscribers:
            self.messages.append(message)
        else:
            target = self.subscribers.pop(0)
            target.send(self.prep_message(message))
            self.subscribers.append(target)
    
    def empty(self):
        return not bool(self.subscribers or self.messages)
    

class Topic(MessageQueueRoot):
    def send(self, message):
        """ Calls the send function for each attached protocol """
        for proto in self.subscribers:
            proto.send(self.prep_message(message))
  

class Pubsub(object):
    """
    Sending to a pubsub queue replicates the message to all subordinate queues. 
    Each pubsub queue contains the list of message queues that should
    receive the message.
    Clients cannot subscribe to the pubsub queue itself, they must subscribe
    to a subordinate queue.
    """
    def __init__(self, name):
        self.baseQueue = Queue(name)
        self.child_queues = []
        
    def add_child(self, queue_ref):
        if queue_ref not in self.child_queues:
            self.child_queues.append(queue_ref)
            
    def remove_child(self, queue_ref):
        if queue_ref in self.child_queues:
            self.child_queues.remove(queue_ref)

    def subscribe(self, proto):
        raise QueueError("FAIL")
            
    def send(self, message):
        """ Calls the send function for each child queue """
        for q in self.child_queues:
            q.send(message)
    
    def empty(self):
        return not bool(len(self.child_queues) > 0)
    

class MessageQueueManager(object):

    def __init__(self):
        self.message_queues = {}
        
    def set_queue_rights(self, queue_rights):
        self.queue_rights = queue_rights

    def test_queue_rights(self, groups, qname, rights):
        if rights not in self.queue_rights(groups, qname):
            error = "FAIL"+rights.upper()
            raise QueueError(error)
    
    def create_queue(self, proto, dest_name):
        if self.message_queues.has_key(dest_name):
            return
        self.test_queue_rights(proto.get_groups(), dest_name, 'c')
        if dest_name.startswith('/queue/'):
            dest = Queue(dest_name)
        elif dest_name.startswith('/pubsub/'):
            dest = Pubsub(dest_name)
        else: dest = Topic(dest_name)
        self.message_queues[dest_name] = dest
    
    def subscribe_queue(self, proto, dest_name):
        self.create_queue(proto, dest_name)
        self.test_queue_rights(proto.get_groups(), dest_name, 'r')
        self.message_queues[dest_name].subscribe(proto)
    
    def send_message(self, proto, dest_name, message):
        #print "mqm - send - P, D, M", proto, dest_name, message
        self.create_queue(proto, dest_name)
        self.test_queue_rights(proto.get_groups(), dest_name, 'w')
        if not isinstance(message, tuple):
            message = ({}, message)
        self.message_queues[dest_name].send(message)
        
    def leave_queue(self, proto, dest_name):
        if self.message_queues.has_key(dest_name):
            self.message_queues[dest_name].unsubscribe(proto)
            
    def destroy_queue(self, proto, dest_name):
        self.create_queue(proto, dest_name)
        #self.test_queue_rights(proto.get_groups(), dest_name, 'c')
        # Go through all the pubsub queues
        self.remove_pubsub_child(self.message_queues[dest_name])
        del self.message_queues[dest_name]
        
    def remove_pubsub_child(self, queue_ref):
        [mq.remove_child(queue_ref) for mq in message_queues 
            if isinstance(mq, Pubsub)]

    def add_pubsub_child(self, proto, dest_name, child_queue):
        self.create_queue(proto, dest_name)
        self.test_queue_rights(proto.get_groups(), child_queue, 'w')
        self.create_queue(proto, child_queue)
        self.message_queues[dest_name].add_child(self.message_queues[child_queue])
    
    def get_list_of_queues(self):
        aRoomList = self.message_queues.keys()
        aRoomList.sort()
        return aRoomList

    def unsubscribe_all_queues(self, proto):
        for q_name in self.message_queues:
            self.message_queues[q_name].unsubscribe(proto)
        



class Parms(object):
    def checker_config(self):
        return {'sec_type':'test'}

    def group_config(self):
        return lambda name: [name]
    
    def get_group_access_rights(self):
        return self.queue_rights
    
    def queue_rights(self, groups, queue):
    #Simple access rights test.
    #Admin group automatically has all rights - no need to check anything else
    #A group has the access specified by the name of the queue
    #/queue/group1/c/r/w/ - Group 1 has all rights
    #/queue/group2/r/ - Group 2 has read-only access
        r = set()
        for g in groups:
            if g == 'admin':
                r = set(['c','r','w'])
                break
            if g not in queue:
                continue
            for type in ['/c/', '/r/', '/w/']:
                if type in queue:
                    r.add(type[1])
        return r


import unittest

from morbid import messagequeue

class TestAvatar(object):
    def __init__(self, user_name, groups):
        self.user_name = user_name
        self.groups = groups
        
class TestProtocol(object):
    def __init__(self, avatar):
        self.num_msg_sent = 0
        self.avatar = avatar
        
    def send(self, message):
        #print "Message sent : ", message
        self.num_msg_sent += 1
        self.headers = message[0]
        self.body = message[1]

    def get_groups(self):
        return self.avatar.groups
    
def queue_rights(group_list, qname):
    #Simple access rights test.
    #Admin group automatically has all rights - no need to check anything else
    #A group has the access specified by the name of the queue
    #/queue/group1/c/r/w/ - Group 1 has all rights
    #/queue/group2/r/ - Group 2 has read-only access
    r = set()
    for g in group_list:
        if g == 'admin':
            r = set(['c','r','w'])
            break
        if g not in qname:
            continue
        for type in ['/c/', '/r/', '/w/']:
            if type in qname:
                r.add(type[1])
    return r

# Useful constants for the tests

uall = '/topic/users/c/r/w/'
uread = '/topic/users/r/'
uwrite = '/topic/users/w/'

qall = '/queue/users/c/r/w/'
qread = '/queue/users/r/'
qwrite = '/queue/users/w/'

pall = '/pubsub/users/c/r/w/'

test_message =  "This is a test"
test_message_2 = "This is line 2"
test_header_1 = {'transaction':'atran'}
    
class MQMTestCase(unittest.TestCase):
    def setUp(self):
        self.a1 = TestProtocol(TestAvatar('tskww', ['users','admin']))
        self.u1 = TestProtocol(TestAvatar('usr1', ['users']))
        self.u2 = TestProtocol(TestAvatar('usr2', ['users']))
        self.u3 = TestProtocol(TestAvatar('usr3', ['none']))
        self.mqm = messagequeue.MessageQueueManager()
        self.mqm.set_queue_rights(queue_rights)
        
    def testCreateTopic(self):
        self.mqm.create_queue(self.u1, uall)
        self.assert_(self.mqm.message_queues.has_key(uall))
        self.assert_(isinstance(self.mqm.message_queues[uall], messagequeue.Topic))
         
    def testFailCreateTopic(self):
        self.assertRaises(messagequeue.QueueError, self.mqm.create_queue, self.u1, uread)
        
    def testSubscribeTopic(self):
        self.mqm.create_queue(self.u1, uall)
        self.mqm.subscribe_queue(self.u2, uall)
        self.assertEquals(self.mqm.message_queues[uall].subscribers[0], self.u2)

    def testFailSubscribeTopic(self):
        self.mqm.create_queue(self.u1, uall)
        self.assertRaises(messagequeue.QueueError, self.mqm.subscribe_queue, self.u3, uall)

    def testAdminCreateTopic(self):
        self.mqm.create_queue(self.a1, uread)
        self.assert_(self.mqm.message_queues.has_key(uread))

    def testCreateQueue(self):
        self.mqm.create_queue(self.u1, qall)
        self.assert_(self.mqm.message_queues.has_key(qall))
        self.assert_(isinstance(self.mqm.message_queues[qall], messagequeue.Queue))
        
    def testFailCreateQueue(self):
        self.assertRaises(messagequeue.QueueError, self.mqm.create_queue, self.u1, qread)

    def testAdminCreateQueue(self):
        self.mqm.create_queue(self.a1, qread)
        self.assert_(self.mqm.message_queues.has_key(qread))
        
    def testSubscribeQueue(self):
        self.mqm.create_queue(self.u1, qall)
        self.mqm.subscribe_queue(self.u2, qall)
        self.assertEquals(self.mqm.message_queues[qall].subscribers[0], self.u2)

    def testFailSubscribeQueue(self):
        self.mqm.create_queue(self.u1, qall)
        self.assertRaises(messagequeue.QueueError, self.mqm.subscribe_queue, self.u3, qall)
        

class TopicTestCase(unittest.TestCase):
    def setUp(self):
        self.u1 = TestProtocol(TestAvatar('usr1', ['users']))
        self.u2 = TestProtocol(TestAvatar('usr2', ['users']))
        self.u3 = TestProtocol(TestAvatar('usr3', ['none']))
        self.mqm = messagequeue.MessageQueueManager()
        self.mqm.set_queue_rights(queue_rights)
        self.mqm.create_queue(self.u1, uall)
        self.mqm.subscribe_queue(self.u2, uall)
        
    def testSendMessage(self):
        self.u4 = TestProtocol(TestAvatar('usr4', ['users']))
        self.mqm.subscribe_queue(self.u4, uall)
        self.mqm.send_message(self.u1, uall, test_message)
        self.assertEquals(self.u2.num_msg_sent, 1)
        self.assertEquals(self.u2.body, test_message)
        self.assertEquals(self.u4.num_msg_sent, 1)
        self.assertEquals(self.u4.body, test_message)
        
    def testFailSendMessage(self):
        self.assertRaises(messagequeue.QueueError, self.mqm.send_message, self.u3, uall, test_message)
        
    def testHeaderMaintenance(self):
        self.mqm.subscribe_queue(self.u2, uall)
        self.mqm.send_message(self.u1, uall, (test_header_1, test_message))
        self.assertEquals(self.u2.headers['transaction'], 'atran')


class QueueTestCase(unittest.TestCase):
    def setUp(self):
        self.u1 = TestProtocol(TestAvatar('usr1', ['users']))
        self.u2 = TestProtocol(TestAvatar('usr2', ['users']))
        self.u3 = TestProtocol(TestAvatar('usr3', ['none']))
        self.mqm = messagequeue.MessageQueueManager()
        self.mqm.set_queue_rights(queue_rights)
        self.mqm.create_queue(self.u1, qall)

    def testSendMessage(self):
        self.mqm.subscribe_queue(self.u2, qall)
        self.mqm.send_message(self.u1, qall, test_message)
        self.assertEquals(self.u2.num_msg_sent, 1)
        self.assertEquals(self.u2.body, test_message)

    def testFailSendMessage(self):
        self.assertRaises(messagequeue.QueueError, self.mqm.send_message, self.u3, qall, test_message)

    def testQueueMessage(self):
        self.mqm.send_message(self.u1, qall, test_message)
        self.assertEquals(len(self.mqm.message_queues[qall].messages), 1)
        self.assertEquals(self.mqm.message_queues[qall].messages[0][0], {})
        self.assertEquals(self.mqm.message_queues[qall].messages[0][1], test_message)
        
    def testReadQueuedMessage(self):
        self.mqm.send_message(self.u1, qall, test_message)
        self.mqm.subscribe_queue(self.u2, qall)
        self.assertEquals(len(self.mqm.message_queues[qall].messages), 0)
        self.assertEquals(self.u2.body, test_message)
        
    def testTwoQueuedMessages(self):
        self.mqm.send_message(self.u1, qall, test_message)
        self.mqm.send_message(self.u1, qall, (test_header_1, test_message_2))
        self.assertEquals(len(self.mqm.message_queues[qall].messages), 2)
        self.assertEquals(self.mqm.message_queues[qall].messages[0][1], test_message)
        self.assertEquals(self.mqm.message_queues[qall].messages[1][0]['transaction'], 'atran')
        self.assertEquals(self.mqm.message_queues[qall].messages[1][1], test_message_2)

    def testReadTwoQueuedMessages(self):
        self.mqm.send_message(self.u1, qall, test_message)
        self.mqm.send_message(self.u1, qall, test_message_2)
        self.mqm.subscribe_queue(self.u2, qall)
        self.assertEquals(len(self.mqm.message_queues[qall].messages), 0)
        self.assertEquals(self.u2.num_msg_sent, 2)
        self.assertEquals(self.u2.body, test_message_2)

    def testTwoReadTwoQueuedMessages(self):
        self.u3 = TestProtocol(TestAvatar('usr3', ['user']))
        self.mqm.subscribe_queue(self.u2, qall)
        self.mqm.subscribe_queue(self.u3, qall)
        self.mqm.send_message(self.u1, qall, test_message)
        self.mqm.send_message(self.u1, qall, test_message_2)
        self.assertEquals(len(self.mqm.message_queues[qall].messages), 0)
        self.assertEquals(self.u2.num_msg_sent, 1)
        self.assertEquals(self.u2.body, test_message)
        self.assertEquals(self.u3.num_msg_sent, 1)
        self.assertEquals(self.u3.body, test_message_2)
        
    def testQueueHeaderMaintenance(self):
        self.mqm.subscribe_queue(self.u2, qall)
        self.mqm.send_message(self.u1, qall, (test_header_1, test_message))
        self.assertEquals(self.u2.headers['transaction'], 'atran')


class PubsubTestCase(unittest.TestCase):
    def setUp(self):
        self.u1 = TestProtocol(TestAvatar('usr1', ['users']))
        self.u2 = TestProtocol(TestAvatar('usr2', ['users']))
        self.u3 = TestProtocol(TestAvatar('usr3', ['users']))
        self.u4 = TestProtocol(TestAvatar('usr4', ['users']))
        self.mqm = messagequeue.MessageQueueManager()
        self.mqm.set_queue_rights(queue_rights)

        self.mqm.create_queue(self.u1, uall+"1")
        self.mqm.create_queue(self.u1, uall+"2")
        self.mqm.create_queue(self.u1, uall+"3")

        self.mqm.subscribe_queue(self.u2, uall+"1")
        self.mqm.subscribe_queue(self.u3, uall+"2")
        self.mqm.subscribe_queue(self.u4, uall+"3")
        
    def testCreatePubsub(self):
        self.mqm.create_queue(self.u1, pall)
        self.mqm.add_pubsub_child(self.u1, pall, uall+"1")
        self.mqm.add_pubsub_child(self.u1, pall, uall+"2")
        self.mqm.add_pubsub_child(self.u1, pall, uall+"3")
        
        self.mqm.send_message(self.u1, pall, (test_header_1, test_message))
        
        self.assertEquals(self.u2.body, test_message)
        self.assertEquals(self.u3.body, test_message)
        self.assertEquals(self.u4.body, test_message)



from twisted.cred import portal, checkers, credentials
from twisted.internet import defer
from zope.interface import Interface, implements

# This is a collection of classes used to provide authentication services
# within the twisted framework.

# Copyright (c) 2008 Kendall Whitesell
# Please see the LICENSE file in this package for terms of use


class IConnector(Interface):
    "Attributes: user_name, (group names)"


class Connector(object):
    implements(IConnector)
    def __init__(self, user_name, groups):
        self.user_name = user_name
        self.groups = groups
        #print "Connector", self.user_name, self.groups


class MQRealm(object):
    implements(portal.IRealm)
    
    def __init__(self, parms):
        try:
            self.get_groups = parms.group_config()
        except:
            self.get_groups = lambda name: "NoGroup"
            
    def requestAvatar(self, avatarID, mind, *interfaces):
        assert IConnector in interfaces
        logout = lambda: None
        groups = self.get_groups(avatarID)
        return (IConnector, Connector(avatarID, groups), logout)
    

class MQDefaultParms(object):
    def checker_config(self):
        return {'sec_type':'any'}

    def group_config(self):
        return lambda name: [name]
    
    def get_group_access_rights(self):
        return self.get_group_rights
    
    def get_group_rights(self, groups, queue):
        return set(['c','r','w'])


class MQPortal(object):
    def __init__(self, mqm, filename=None):
        if filename:
            parms = self.process_config_file(filename)
        else:
            parms = MQDefaultParms()
        kw = parms.checker_config()
        checker_types = { 'test': self.create_memory_checker,
                          'file': self.create_file_checker,
                          'pam': self.create_pam_checker,
                          'db': self.create_database_checker,
                          'any': self.accept_all
                          }
        #print "Checker_type", checker_type
        self.p = portal.Portal(MQRealm(parms))
        checker = checker_types.get(kw.get('sec_type',''), self.accept_all)(**kw)
        self.p.registerChecker(checker)
        mqm.set_queue_rights(parms.get_group_access_rights())
        
    def accept_all(self, **kw):
        accept_any = AcceptAllIdChecker()
        return accept_any
            
    def create_memory_checker(self, **kw):
        my_checker = checkers.InMemoryUsernamePasswordDatabaseDontUse()
        for x in xrange(kw.get('id_count', 10)):
            my_checker.addUser("usr%d"%x, "pwd%d"%x)
        return my_checker
    
    def create_file_checker(self, **kw):
        file_db_parms = {}
        for parm in ['delim', 'usernameField', 'passwordField', 'caseSensitive', 'hash', 'cache']:
            if kw.has_key(parm): file_db_parms[parm]=kw[parm]
        #print file_db_parms
        my_checker = checkers.FilePasswordDB(kw['filename'], **file_db_parms)
        return my_checker
    
    def create_pam_checker(self, **kw):
        #TODO: test this
        return checkers.PluggableAuthenticationModulesChecker()

    def create_database_checker(self, **kw):
        #TODO: implement this
        "This is where a database connection would be hooked-up"

    def stomp_login(self, **kw):
        #print "In stomp_login", kw.get("login","None"), kw.get("passcode", "None")
        creds = credentials.UsernamePassword(kw.get("login", ""),
                                             kw.get("passcode", ""))
        d = self.p.login(creds, None, IConnector)
        #print "Stomp_login d = ", d
        return d

    def process_config_file(self, filename):
        if filename:
            try:
                import os, sys
                path, fname = os.path.split(filename)
                sys.path.insert(0, path)
                globals()['security_parameters'] = __import__(fname) 
            except:
                print "Error in import of security data from file:",filename
                return None
        else:
            from morbid.sample_auth import mqanyconf as security_parameters
        return security_parameters.Parms()

class AcceptAllIdChecker(object):
    implements(checkers.ICredentialsChecker)
    credentialInterfaces = (credentials.IUsernamePassword,)
    
    def requestAvatarId(self, credentials):
        #print "AcceptAll requestAvatarId", credentials, credentials.username
        uname = credentials.username
        return defer.succeed(uname)
    
    


from twisted.internet import defer
from twisted.web.client import getPage
from error import error

try:
    try:
        import cjson as json
    except ImportError:
        try:
            import simplejson as json
        except ImportError:
            import json
except ImportError:
    raise ImportError, "Could not load one of: Python json, cjson, simplejson. Please install a json module."
if hasattr(json, 'encode'):
    encode = json.encode
    decode = json.decode
elif hasattr(json, 'dumps'):
    encode = json.dumps
    decode = json.loads
elif hasattr(json, 'write'):
    encode = json.write
    decode = json.read
else:
    raise ImportError, 'Fatal Error: loaded unknown json module: "%s"'%(json.__file__,)
"""
This fixes the unicode problem for now.
It would be more efficient to do the
unicode check early on and try to use
a json module that doesn't require this
hack (for now, I'm just trying them in
what I consider a reasonable order). It
also might be nice to have other json
options (I removed demjson because it
was somehow still causing unicode errors).

-mario
"""
if not isinstance(decode(encode('test')), str):
    _decode = decode
    def decode(data):
        def fix_unicode(chunk):
            if isinstance(chunk, list):
                for i in range(len(chunk)):
                    chunk[i] = fix_unicode(chunk[i])
            elif isinstance(chunk, dict):
                old_chunk = chunk
                chunk = {}
                for key, val in old_chunk.items():
                    chunk[str(key)] = fix_unicode(val)
            elif isinstance(chunk, unicode):
                chunk = str(chunk)
            return chunk
        return fix_unicode(_decode(data))

def eb(url):
    def _eb(e):
        error('problem connecting to server: "%s"'%(url,))
    return _eb

class RestQ(object):
    def __init__(self, rqaddr):
        self.cbs = {
            'connect':None,
            'disconnect':None,
            'subscribe':None,
            'unsubscribe':None,
            'send':None
        }
        if rqaddr:
            getPage(rqaddr).addCallback(self.initialize).addErrback(eb(rqaddr))

    def submit(self, conn, cmd, headers={}, body=""):
        url = self.cbs.get(cmd,None)
        if url is not None:
            headers["username"] = conn.username
            if cmd == "send":
                headers['body'] = body
            def cb(raw_data):
                data = decode(raw_data)
                if "allow" in data and data["allow"] == "no":
                    return (data, None)
                newBody = data.get('body',body)
                headers['content-length'] = len(newBody)
                if "autosubscribe" in data:
                    conn.autosubscribe(data['autosubscribe'])
                if "autounsubscribe" in data:
                    conn.autounsubscribe(data['autounsubscribe'])
                return (headers, newBody)
            sendDeferred = getPage(url, method='POST', postdata=encode(headers)).addCallback(cb).addErrback(eb(url))
            if cmd in ["connect", "subscribe", "send"]:
                return sendDeferred
        return defer.succeed((headers, body))

    def initialize(self, rawData):
        data = decode(rawData)
        for key, value in data.items():
            self.cbs[key] = value



from twisted.web import server, resource
from twisted.internet import reactor
import time
try:
    try:
        import cjson as json
    except ImportError:
        try:
            import simplejson as json
        except ImportError:
            import json
except ImportError:
    raise ImportError, "Could not load one of: Python json, cjson, simplejson. Please install a json module."
if hasattr(json, 'encode'):
    encode = json.encode
    decode = json.decode
elif hasattr(json, 'dumps'):
    encode = json.dumps
    decode = json.loads
elif hasattr(json, 'write'):
    encode = json.write
    decode = json.read
else:
    raise ImportError, 'Fatal Error: loaded unknown json module: "%s".'%(json.__file__,)

cbUrls = None

class DummyLeaf(object):
    def __init__(self, data):
        self.data = data

    def render(self, request):
        return self.data

def wrap(data):
    print "sending data:",data
    return DummyLeaf(encode(data))

class RestQDummyResource(resource.Resource):
    def getChild(self, path, request):
        if not path or path == "/":
            return wrap(cbUrls)
        headers = decode(request.content.read())
        print '-------------------'
        print 'headers:',headers
        username = headers['username']
        destination = headers.get("destination",None)
        if path in cbUrls:
            print 'checking path: "%s"'%(path,)
            if path == "connect":
                if username == "noconnect":
                    return wrap({"allow":"no"})
            elif path == "send":
                newBody = headers['body'].replace("apples","bananas")
                if username == 'nosend':
                    return wrap({"allow":"no"})
                elif username == 'slow':
                    time.sleep(1)
                return wrap({"body":newBody})
            elif path == "subscribe":
                if destination == "auto":
                    return wrap({"autosubscribe":["room1","room2","room3"]})
            elif path == "unsubscribe":
                if destination == "auto":
                    return wrap({"autounsubscribe":["room1","room2","room3"]})
            return wrap({})

def main():
    global cbUrls
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option(
        "-p",
        "--port",
        dest="port",
        type="int",
        default=5000,
        help="port the RestQ demonstration resource should bind to. default: 5000"
    )
    options = parser.parse_args()[0]
    cbUrls = {
        'connect':'http://localhost:%s/connect'%options.port,
        'disconnect':'http://localhost:%s/disconnect'%options.port,
        'subscribe':'http://localhost:%s/subscribe'%options.port,
        'unsubscribe':'http://localhost:%s/unsubscribe'%options.port,
        'send':'http://localhost:%s/send'%options.port
    }
    site = server.Site(RestQDummyResource())
    print 'running on %s'%options.port
    reactor.listenTCP(options.port, site)
    reactor.run()

if __name__ == "__main__":
    main()


def error(msg, reactor_running=True):
    print "MorbidQ Error:",msg
    if reactor_running:
        from twisted.internet import reactor
        reactor.stop()
    else:
        import sys
        sys.exit(-1)


from setuptools import setup, find_packages

setup(
    name='morbid',
    version='0.8.7.2',
    author='Michael Carter',
    author_email='CarterMichael@gmail.com',
    license='MIT License',
    install_requires = ['stomper'],
    description='A Twisted-based publish/subscribe messaging server that uses the STOMP protocol',
    packages= find_packages(),
    entry_points = '''
        [console_scripts]
        morbid = morbid:main
        morbid_restq_test = morbid.sample_restq.restq_dummy_daemon:main
    ''',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



import traceback
import stomper
import mqsecurity
from datetime import datetime
from restq import RestQ
from messagequeue import QueueError
from error import error

verbose = False

try:
    from twisted.internet.protocol import Factory, Protocol
except:
    error("Twisted required to run; see www.twistedmatrix.com", False)

class StompProtocol(Protocol):
    id = 0
    def __init__(self):
        self.state = 'initial'
        self.username = None
        self.password = None
        self.buffer = ""
        self.stompBuffer = stomper.stompbuffer.StompBuffer()
        self.lastNull = False
        StompProtocol.id += 1
        self.id = StompProtocol.id
        self.avatar = None
        self.receivedDisconnect = False

    def get_groups(self):
        return self.avatar.groups

    def dataReceived(self, data):
        # NOTE: Allow each frame to have an optional '\n'
        # NOTE: binary DOES NOT WORK with this hack in place
        self.stompBuffer.appendData(data.replace('\0', '\0\n'))
        while True:
            msg = self.stompBuffer.getOneMessage()
            # NOTE: the rest of the optional '\n' hack
            if self.stompBuffer.buffer.startswith('\n'):
                self.stompBuffer.buffer = self.stompBuffer.buffer[1:]
            if msg is None:
                break
            if not msg['headers'] and not msg['body'] and not msg['cmd']:
                break
            msg['cmd'] = msg['cmd'].lower()
            getattr(self, 'read_%s' % self.state)(**msg)

    def sendError(self, e):
        exception, instance, tb = traceback.sys.exc_info()
        tbOutput= "".join(traceback.format_tb(tb))
        self.sendFrame('ERROR', {'message': str(e) }, tbOutput)

    def sendFrame(self, cmd, headers, body):
        f = stomper.Frame()
        f.cmd = cmd
        f.headers.update(headers)
        f.body = body
        self.transport.write(f.pack())

    def autosubscribe(self, subs):
        for sub in subs:
            self.frame_subscribe(({'destination':sub}, None))

    def autounsubscribe(self, unsubs):
        for unsub in unsubs:
            self.frame_unsubscribe(({'destination':unsub}, None))

    def read_initial(self, cmd, headers, body):
        assert cmd == 'connect', "Invalid cmd: expected CONNECT"
        self.username = headers.get('login',"")
        self.password = headers.get('passcode',"")
        d = None
        if self.factory.mq_portal:
            d = self.factory.mq_portal.stomp_login(**headers)
        if d:
            d.addCallback(self.stomp_connected).addErrback(self.stomp_connect_failed)
        else:
            self.stomp_connected((IConnector, Connector("None", ["None"]), lambda: None))

    def stomp_connected(self, *args):
        def stomp_connected_callback((headers, body)):
            if "allow" in headers and headers["allow"] == "no":
                return self.stomp_connect_failed()
            avatarInterface, avatar, logout_func = args[0]
            self.avatar = avatar
            self.state = 'connected'
            self.sendFrame('CONNECTED', {"session": self.id}, "")
        self.factory.restq.submit(self, "connect", {"username":self.username, "password":self.password}).addCallback(stomp_connected_callback)

    def stomp_connect_failed(self, *args):
        self.sendFrame('ERROR', {'message': "Invalid ID or password"}, "Invalid ID or password")
        self.transport.loseConnection()

    def read_connected(self, cmd, headers, body):
        self.factory.restq.submit(self, cmd, headers, body).addCallback(getattr(self, 'frame_%s' % cmd))

    def frame_subscribe(self, (headers, body)):
        if "allow" in headers and headers["allow"] == "no":
            return
        try:
            self.factory.mqm.subscribe_queue(self, headers['destination'])
        except QueueError, err:
            self.sendFrame('ERROR',
                           {'message': self.get_message_code(err.code)},
                           self.get_message_text(err.code))

    def frame_unsubscribe(self, (headers, body)):
        self.factory.mqm.leave_queue(self, headers['destination'])

    def frame_send(self, (headers, body)):
        if "allow" in headers and headers["allow"] == "no":
            return
        try:
            result = self.factory.mqm.send_message(self, headers['destination'], (headers, body))
        except QueueError, err:
            self.sendFrame('ERROR',
                           {'message': self.get_message_code(err.code)},
                           self.get_message_text(err.code))

    def frame_disconnect(self, (headers, body)):
        self.receivedDisconnect = True
        self.transport.loseConnection()

    def connectionLost(self, reason):
        if not self.receivedDisconnect and self.username:
            self.factory.restq.submit(self, "disconnect")
        self.factory.disconnected(self)

    def get_message_code(self, code):
        return {'FAILC': "CREATE error",
                'FAILR': "READ error",
                'FAILW': "WRITE error"}[code]

    def get_message_text(self, code):
        return {'FAILC': "Not authorized to create queue",
                'FAILR': "Not authorized to read queue",
                'FAILW': "Not authorized to write to queue"}[code]

    def send(self, message):
        '''
        This method is invoked by the message queues.
        Not intended for direct use by the protocol.
        '''
        headers, body = message
        self.sendFrame('MESSAGE', headers, body)

class StompFactory(Factory):
    """
    The StompFactory creates an instance of a StompProtocol for each connection.
    Successful authentication results in the creation of an avatar for that user.
    The Avatar is assigned to the StompProtocol.
    """
    protocol = StompProtocol

    def __init__(self, mqm=None, filename=None, rqaddr=None, verbose=False):
        self.id = 0
        self.restq = RestQ(rqaddr)
        self.verbose = verbose
        if mqm:
            self.mqm = mqm
        else:
            import messagequeue
            self.mqm = messagequeue.MessageQueueManager()
        self.mq_portal = mqsecurity.MQPortal(self.mqm, filename=filename)

    def report(self, msg):
        if self.verbose:
            print "[%s] MorbidQ: %s"%(datetime.now(), msg)

    def disconnected(self, proto):
        self.mqm.unsubscribe_all_queues(proto)

def load_config(config_file, config_dict):
    import os
    if not os.path.isfile(config_file):
        return error('config file "%s" does not exist'%(config_file,))
    f = open(config_file)
    lines = f.readlines()
    f.close()
    for line in lines:
        if "=" not in line:
            return error('error parsing config. line: "%s"'%(line,))
        option, value = line.split("=",1)
        if option == "verbose":
            if value.lower() == "true":
                value = True
            else:
                value = False
        if option == "port":
            try:
                value = int(value)
            except:
                return error('non-integer port specified: "%s"'%(line,))
        config_dict[option.strip()] = value.strip()

def get_stomp_factory(config_file="", cfg={'auth':None,'restq':None,'verbose':False}):
    if config_file:
        load_config(config_file, cfg)
    f = StompFactory(filename=cfg['auth'], rqaddr=cfg['restq'], verbose=cfg['verbose'])
    f.report("server starting with configuration: %s"%(cfg,))
    return f

def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option(
        "-c",
        "--config",
        dest="config",
        type="string",
        default="",
        help="configuration file"
    )
    parser.add_option(
        "-p",
        "--port",
        dest="port",
        type="int",
        default=61613,
        help="port the daemon should bind to (default: 61613)"
    )
    parser.add_option(
        "-i",
        "--interface",
        dest="interface",
        type="string",
        default="",
        help="hostname the daemon should bind to (default: all interfaces)"
    )
    parser.add_option(
        "-a",
        "--auth",
        dest="auth",
        type="string",
        default="",
        help="local path to authentication module"
    )
    parser.add_option(
        "-r",
        "--restq",
        dest="restq",
        type="string",
        default="",
        help="RestQ http callback resource"
    )
    parser.add_option(
        "-v",
        "--verbose",
        dest="verbose",
        action="store_true",
        default=False,
        help="verbose mode"
    )
    from twisted.internet import reactor
    (options, args) = parser.parse_args()
    stomp_factory = get_stomp_factory(options.config, {
        'verbose':options.verbose,
        'restq':options.restq,
        'auth':options.auth
    })
    reactor.listenTCP(options.port, stomp_factory, interface=options.interface)
    reactor.run()

if __name__ == "__main__":
    main()
